<?php
namespace Service;

class GetOrigins {
    public function manageOrigin($IDORIGEMVENDA, $type, $model) {
        // type: sistema ou impressao
        // model: modalidade ou origem

        $typeMap = [
            "system" => [
                "origin" => "getSystemOrigin",
                "modality" => "getSystemModality",
                "deliveryOrigin" => "getDeliveryOrigin"
            ],
            "print" => [
                "origin" => "getPrintOrigin",
                "modality" => "getPrintModality"
            ]
        ];
    
        $method = $typeMap[$type][$model];
        return $this->$method($IDORIGEMVENDA);
    }

    private function searchArrayKeyVal($array, $origin, $default) {
        // procura a origem no array de origens/modalidades possíveis e retorna o valor correspondente ou o valor default

        $foundKey = $default;
        foreach ($array as $key => $value) {
            if (strpos($origin, $key) !== false) {
                $foundKey = $value;
                break;
            }
        }
        if ($foundKey == "Delivery") {
            $foundKey = $this->getDeliveryOrigin($origin);
        }
        return $foundKey;
    }

    private function getPrintOrigin($IDORIGEMVENDA) {
        $possibleOrigins = array (
            'TAA' => 'Totem',
            'RAP' => 'Rappi',
            'UBR' => 'Uber Eats',
            'APP' => 'APP',
            'FOS' => 'For Sale',
            'MOB' => 'POS'
        );

        return $this->searchArrayKeyVal($possibleOrigins, $IDORIGEMVENDA, 'APP');          
    }

    private function getPrintModality($IDORIGEMVENDA) {
        $possibleModalities = array (
            'DLV' => 'Delivery',
            'THR' => 'Drive',
            'TGO' => 'Viagem',
            'BAL' => 'Balcão',
            'CMD' => 'Comanda',
            'MES' => 'Mesa'
        );

        return $this->searchArrayKeyVal($possibleModalities, $IDORIGEMVENDA, 'Balcão');
    }

    private function getSystemOrigin($IDORIGEMVENDA) {
        $possibleOrigins = array (
            'DLV' => 'Delivery',
            'THR' => 'Drive',
            'TGO_TAA' => 'TAA Viagem',
            'TGO' => 'Viagem',
            'TAA' => 'TAA',
            'MES_MOB' => 'MesaPOS',
            'BAL_AUT' => 'TAA'
        );

        return $this->searchArrayKeyVal($possibleOrigins, $IDORIGEMVENDA, 'Balcão');
    }

    private function getSystemModality($IDORIGEMVENDA) {
        $possibleModalities = array (
            'DLV' => "DLV",
            'THR' => "Drive",
            'TGO' => "Viagem",
            'BAL_AUT' => 'TAA',
            'TAA' => 'TAA',
            'BAL' => "Balcão",
            'CMD' => "Comanda",
            'MES_MOB' => "MesaPOS",
            'MES' => "Mesa"
        );
        
        return $this->searchArrayKeyVal($possibleModalities, $IDORIGEMVENDA, $IDORIGEMVENDA);
    }

    private function getDeliveryOrigin ($IDORIGEMVENDA) {

        $possibleModalities = array (
            'DLV_FOS' => "DELIVERY LOJA",
            'DLV_FNC' => "DELIVERY LOJA",
            'DLV_SAT' => "DELIVERY LOJA",
            'DLV_IFO' => 'DELIVERY IFOOD',
            'DLV_PJA' => 'DELIVERY PEDIDOS JÁ',
            'DLV_SIT' => "DELIVERY SITE",
            'DLV_CAL' => "DELIVERY CALL CENTER",
            'DLV_INT' => "DELIVERY INTEGRAÇÃO",
            'DLV_SPO' => "DELIVERY SPOON ROCKET",
            'DLV_APP' => "DELIVERY APP",
            'DLV_ADR' => "DELIVERY ANDROID",
            'DLV_IOS' => "DELIVERY IOS",
            'DLV_ABF' => "DELIVERY ABRAFOOD",
            'DLV_ONY' => "DELIVERY PEDE PRONTO",
            'DLV_WZP' => "DELIVERY WHATSAPP",
            'DLV_YAN' => "DELIVERY YANDEH",
            'DLV_UBR' => "DELIVERY UBER EATS",
            'DLV_RAP' => "DELIVERY RAPPI",
            'DLV_99F' => "DELIVERY 99FOOD",
            'DLV_JAM' => "DELIVERY JAMES",
            'DLV_4AL' => "DELIVERY 4ALL",
            'DLV_ORD' => "DELIVERY",
            'DLV_NWB' => "DELIVERY NEEMO",
            'DLV_NAD' => "DELIVERY ANDROID",
            'DLV_NOS' => "DELIVERY IOS",
            'DLV_NMO' => "DELIVERY NEEMO",
            'DLV_NBT' => "DELIVERY NEEMO",
            'DLV_SID' => "DELIVERY",
            'DLV_DDI' => "DELIVERY DIRETO",
            'DLV_PED' => "DELIVERY PEDE PRONTO",
            'DLV_ALP' => "DELIVERY ALPHACODE"

        );
        
        return $this->searchArrayKeyVal($possibleModalities, $IDORIGEMVENDA, 'DELIVERY');

    }

    public function getToGo ($toGo) {
       $possibleToGo = array(
        'S' => ' - RETIRAR NO BALCÃO',
        'C' => ' - CONSUMIR NO LOCAL'
       );
       return $this->searchArrayKeyVal($possibleToGo, $toGo, '');
    }
}
