<?php
namespace Service;

use LDAP\Result;
use \Remote\Printer\Command;

class Impressao {

	protected $entityManager;
	protected $util;
	protected $impressaoUtil;
	protected $instanceManager;

	public function __construct(
		\Doctrine\ORM\EntityManager $entityManager,
		\Util\Util $util,
		\Lib\ImpressaoUtil $impressaoUtil,
		\Service\GetOrigins $getOrigins,
		\Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager
	){
		$this->entityManager		= $entityManager;
		$this->util					= $util;
		$this->impressaoUtil		= $impressaoUtil;
		$this->getOrigins			= $getOrigins;
		$this->instanceManager = $instanceManager;
	}

	public function imprimePicklist($produtos, $IDIMPPRODAPETIC = null){
		$produtos = $this->getInformacoesProdutos($produtos);
		$params = $this->getParamsProdutos($produtos);
		$MODALIDADE = $this->getOrigins->manageOrigin($params['IDORIGEMVENDA'], "print", "modality");
		$ORIGEM = $this->getOrigins->manageOrigin($params['IDORIGEMVENDA'], "print", "origin");
		$CDCAIXA =  $this->entityManager->getConnection()->fetchAssoc(\Util\Query::GET_CDCAIXA_BY_CDSETOR, array('CDSETOR' => $produtos[0]['CDSETOR']));
		
		
		$CDFILIAL = $params['CDFILIAL'];
		$CDCAIXA = $CDCAIXA['CDCAIXA'];
		$NMCAIXA = $params['NMCAIXA'];
		$NRPEDIDOFOS = $params['NRPEDIDOFOS'];
		$SENHA = $params['CDSENHAPED']; 
		$NRCOMANDA = $params['NRCOMANDA'];
		$DTHORA = $params['DTHORA'];
		$CDLOJA = $params['CDLOJA'];
		$NMLOJA = $params['NMLOJA'];
		$NRVENDAREST = $params['NRVENDAREST'];
		$NRSEQVENDA = $params['NRSEQVENDA'];

		$dadosImpressora = $this->impressaoUtil->getDadosImpressora($CDFILIAL, $CDCAIXA);
	    if (!$dadosImpressora['error'] && $dadosImpressora['dadosImpressora']){
			$comandos = new Command();
			$dadosImpressora = $dadosImpressora['dadosImpressora'];
			$paramsImp = array(
				':CDFILIAL'			=>	$CDFILIAL,
				':CDLOJA'			=>	$CDLOJA, 
				':CDCAIXA'			=>	$CDCAIXA
			);
			
			$r_valImprLoja = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::SQL_VAL_IMPR_LOJA, $paramsImp);
			$IDMODEIMPRES = $r_valImprLoja['IDMODEIMPRES'];
			
			$printerParams = $this->impressaoUtil->buscaParametrosImpressora($IDMODEIMPRES);
			
			$printerParams['letterType'] = $printerParams['tipoLetra'];
			$text = '';
			$text .= $this->impressaoUtil->imprimeLinha($printerParams);
			//colocar string grande e invertida (fundo preto)
			$text .= $this->impressaoUtil->centraliza($printerParams, chr(27).chr(33).chr(48).$MODALIDADE.chr(27).chr(33), $printerParams['largura']).$printerParams['comandoEnter'];
			$text .= $this->impressaoUtil->imprimeLinha($printerParams);
			$text .= $printerParams['comandoEnter'];
			$printerParams['bold'] = true;
			
			$comandos->text($text, $printerParams);
			//abaixo normal
			$text = '';	
			$printerParams['bold'] = false;
			$text .= $this->impressaoUtil->separaComEspacos('Linha 1', $ORIGEM, $printerParams['largura']);
			
			$text .=  $printerParams['comandoEnter'] . $printerParams['comandoEnter'];
			
			// $text .= 'Loja   : ' . $CDLOJA.' - '.$NMLOJA . $printerParams['comandoEnter'];
			// $text .= 'Caixa  : ' . $CDCAIXA.' - '.$NMCAIXA . $printerParams['comandoEnter'];
			
			$comandos->text($text, $printerParams);
			//colocar string grande e invertida (fundo preto)
			$text = '';
			if($SENHA != null){
				$printerParams['bold'] = false;
				$senhaNegrito = chr(27).chr(33).chr(48).$SENHA.chr(27).chr(33); //coloca a senha em negrito e extended, e o texto após a senha normal
				$text .= 'Pedido: ' . $senhaNegrito . $printerParams['comandoEnter'] . $printerParams['comandoEnter'] . substr($DTHORA, 11, 20);
			} else{
				$text .= $this->impressaoUtil->alinhaDireita(substr($DTHORA, 11, 20),  $printerParams['largura']);
			}
				
			$text .=  $printerParams['comandoEnter'] . $printerParams['comandoEnter'];
			$comandos->text($text, $printerParams);
			
			$text = '';
			$printerParams['bold'] = false;
			// produtos em texto normal
			$text .= $this->impressaoUtil->alinhaEsquerda('Produtos  :',  $printerParams['largura']);
			$text .=  $printerParams['comandoEnter'] . $printerParams['comandoEnter'];
				$comandos->text($text, $printerParams);
				
				// nome do produto em negrito e expanded
				$text = chr(27).chr(33).chr(48);
				//$text .= chr(27).chr(77).chr(1);
				$printerParams['italic'] = false;
				$printerParams['bold'] = false;
				$printerParams['expanded'] = false;

			// nome do produto em negrito e expanded
			$text = chr(27).chr(33).chr(48);
			//$text .= chr(27).chr(77).chr(1);
			$printerParams['italic'] = false;
			$printerParams['bold'] = false;
			$printerParams['expanded'] = false;

			//orderna o array de produtos pelo NMPRODUTO
			$name = array_column($produtos, 'NMPRODUTO');
			array_multisort($name, SORT_ASC, $produtos);

			foreach ($produtos as $key => &$produto) {
				$text .= $this->impressaoUtil->alinhaEsquerda($produto['NMPRODUTO'] ,  $printerParams['largura']). $printerParams['comandoEnter'];

				//se o TXPRODPED chegar como string, converte em array
				if(!is_array($produto['TXPRODPED']) && !empty($produto['TXPRODPED'])){
					$decodedTxprodped = json_decode($produto['TXPRODPED']);
					$obsArray = array();
					foreach($decodedTxprodped as $current) {
						array_push($obsArray, get_object_vars($current));
					}

					$produto['TXPRODPED'] = $obsArray;
				}

				//se a observação do TXPRODPED não tiver apelido, coloca o nome da observação
				$observacoesTexto = "";
				$obsName = array();
				if ($produto['TXPRODPED'] != null) {
					foreach($produto['TXPRODPED'] as $key => &$obs){
						if(empty($obs['DSAPELIDOOBS']) || $obs['DSAPELIDOOBS'] == null) {
							$obs['DSAPELIDOOBS'] = $obs['OBSERVACAO'];
						}
						$obs['IDCONTROLAOBS'] = "";
						if(stripos($obs['DSAPELIDOOBS'], "-") !== false){
							$obs['IDCONTROLAOBS'] = '-';
							$obs['DSAPELIDOOBS'] = str_replace("-", "", $obs['DSAPELIDOOBS']);
						};
						if(stripos($obs['DSAPELIDOOBS'], "+") !== false){
							$obs['IDCONTROLAOBS'] = '+';
							$obs['DSAPELIDOOBS'] = str_replace("+", "", $obs['DSAPELIDOOBS']);
						};
					}
	
					// ordena TXPRODPED
					$name = array_column($produto['TXPRODPED'], 'DSAPELIDOOBS');
					array_multisort($name, SORT_ASC, SORT_STRING | SORT_FLAG_CASE, $produto['TXPRODPED']);
					
					if($produto['TXPRODPED'] != null) {
						// array_column($produto['TXPRODPED'], 'DSAPELIDOOBS'); // pega os apelidos das observacoes para imprimir no qrCode
						foreach ($produto['TXPRODPED'] as $key => &$obs) {
							array_push($obsName, $obs['IDCONTROLAOBS'] . $obs['DSAPELIDOOBS']);
						}
						$observacoesTexto = implode(" - ", $obsName); // separa as observacoes por '-'
					}
				}
				$produto['observacoesArray'] = $obsName;
				$produto['observacoesTexto'] = $observacoesTexto;
			}

			//$text .= '1B+21+0';
			$text .= $printerParams['comandoEnter'] . $printerParams['comandoEnter'];

	        $comandos->text($text, $printerParams);

	        $barCodeOptions = array(
                'height'   => 40,
                'width'    => 0,
                'position' => 0,
                'font'     => 0,
                'margin'   => 25
            );

	        if($NRSEQVENDA != ''){
	        	$barCodeOrder = 'h'.$CDCAIXA.$CDFILIAL.$NRSEQVENDA;
	        }else{
	        	$barCodeOrder = 'h'.$CDFILIAL.$NRVENDAREST.$NRCOMANDA;
	        }
            $comandos->barCode($barCodeOrder, $barCodeOptions);
			$comandos->cutPaper();
			
			if($IDIMPPRODAPETIC == 'N' || !$IDIMPPRODAPETIC){
				$result = $this->impressaoUtil->requisicaoPonte($dadosImpressora, $comandos);				
				$result = $this->imprimeEtiquetaProdutos($produtos, $params, $MODALIDADE, $SENHA);
			}else{
				$result = $this->imprimeEtiquetaProdutos($produtos, $params, $MODALIDADE, $SENHA);
			}
	      	
	    }else{
	    	$result['error'] = true;
	    	$result['message'] = 'Não foi possivel obter os dados da impressora.';
	    }
		return $result;
	}

	public function imprimeEtiquetaProdutos($produtos, $params, $MODALIDADE, $SENHA){
		//buscar dados da impressora
		$CDCAIXA =  $this->entityManager->getConnection()->fetchAssoc(\Util\Query::GET_CDCAIXA_BY_CDSETOR, array('CDSETOR' => $produtos[0]['CDSETOR']));
		$CDCAIXA = $CDCAIXA['CDCAIXA'];
		$dadosImpressora = $this->impressaoUtil->getDadosImpressora($params['CDFILIAL'], $CDCAIXA);

		$dadosImpressora = $dadosImpressora['dadosImpressora'];
		$params['CDCAIXA'] = $CDCAIXA; 
		$r_valImprLoja = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::SQL_VAL_IMPR_LOJA, $params);
		$printerParams = $this->impressaoUtil->buscaParametrosImpressora($r_valImprLoja['IDMODEIMPRES']);
		$printerParams['letterType'] = $printerParams['tipoLetra'];
		$printerParams['italic'] = true;
		$printerParams['bold'] = true;
		$printerParams['expanded'] = true;
		
		$comandos = new Command();

		
		foreach ($produtos as $produto) {
			if($produto['IDIMPRPRODPICK'] != 'N'){
				for ($i = 0; $i < $produto['QTPRODPEFOS']; $i++) {
					$qrCodeTextArray = array();
					$qrCodeTextArray['CDFILIAL'] = $params['CDFILIAL'];
					$qrCodeTextArray['CDPRODUTO'] = $produto['CDPRODUTO'];
					$qrCodeTextArray['TXPRODPED'] = $produto['observacoesTexto'];
					$qrCodeTextArray['NMPRODUTO'] = $produto['NMPRODUTO'];
					$qrCodeTextArray['CDARVPROD'] = $produto['CDARVPROD'];
					$qrCodeTextArray['NRPEDIDOFOS'] = $params['NRPEDIDOFOS'];
					$qrCodeTextArray['CDBARPRODUTO'] = $produto['CDBARPRODUTO'];

					$useNewLayoutPicklist = $this->instanceManager->getParameter("useNewLayoutPicklist");
					
					if ($useNewLayoutPicklist) { // imprime no novo layout
						$produto['NMCONSUMIDOR'] = isset($produto['NMCONSUMIDOR']) ? $produto['NMCONSUMIDOR'] : "";
						$text = array(
							"mesa" => $MODALIDADE . ' - ' . $SENHA,
							"nomeCliente" => $produto['NMCONSUMIDOR'],
							"pedido" => $produto['NMPRODUTO'],
							"modificacoes" => $produto['observacoesArray'],
							"QRcode" => json_encode($qrCodeTextArray, JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES)
						);

						$comandos->printESCPOS($text, $printerParams);
						$comandos->cutPaper();
					} else { // imprime no layout antigo
						$text = $MODALIDADE . ' - ' . $SENHA . $printerParams['comandoEnter'] . $printerParams['comandoEnter'];
						$text .= chr(27).chr(33).chr(48);
						if(isset($produto['NMCONSUMIDOR'] )){
							$text .= $this->impressaoUtil->alinhaEsquerda($produto['NMCONSUMIDOR'], $printerParams['largura']);
						}
						
						$text .=  $this->impressaoUtil->alinhaEsquerda($produto['NMPRODUTO'], $printerParams['largura']) . $printerParams['comandoEnter'];
						if($produto['TXPRODPED'] != null){
							foreach($produto['TXPRODPED'] as $key => $obsproduto){
								$APELIDO = empty($obsproduto['DSAPELIDOOBS']) ? $obsproduto['OBSERVACAO'] : $obsproduto['DSAPELIDOOBS'];
								if($APELIDO != ''){
									$text .=  '   '. $obsproduto['IDCONTROLAOBS'] . $APELIDO . $printerParams['comandoEnter'];
								}
							}
						}

						//monta QR Code com informações do produto
						$qrCodeOptions = array(
							'height'   => 40,
							'width'    => 0,
							'position' => 0,
							'font'     => 0,
							'margin'   => 25
						);

						$comandos->text($text, $printerParams);
						$comandos->qrCode(json_encode($qrCodeTextArray, JSON_UNESCAPED_UNICODE), $qrCodeOptions);
						$comandos->cutPaper();
					}
				}
			}
		}

		$result['error'] = false;
        $result = $this->impressaoUtil->requisicaoPonte($dadosImpressora, $comandos);
        return $result;
	}

	private function getParamsProdutos($produtos){
		$produtoBase = $produtos[0];

		$params = array(
				'CDFILIAL' 		=> null,
				'CDCAIXA'  		=> null,
				'NMCAIXA'  		=> null,
				'IDORIGEMVENDA' => null,
				'DSTIPOPEDFOS'  => null,
				'NRPEDIDOFOS'	=> null,
				'NRSEQVENDA'	=> null,
				'DTHORA' 		=> date('d/m/Y H:i:s'),
				'NRCOMANDA'		=> null,
				'CDSENHAPED'	=> null,
				'CDLOJA'		=> null,
				'NMLOJA'		=> null,
				'NRVENDAREST'	=> null
		);

		foreach ($params as $key => $param) {
			if(array_key_exists($key, $produtoBase)){
				$params[$key] = $produtoBase[$key];
			}
		}

		return $params;
	}

	private function getInformacoesProdutos($produtos){
		foreach ($produtos as $key => $produto) {
			$params = array(
				'NRPEDIDOFOS'	=>	$produto['NRPEDIDOFOS'],
				'NRITPEDIDOFOS' =>  $produto['NRITPEDIDOFOS'],
				'CDFILIAL'		=>  $produto['CDFILIAL'],
				'NRVENDAREST'	=>  $produto['NRVENDAREST'],
				'CDPRODUTO' 	=>  $produto['CDPRODUTO'],
				'NRSEQVENDA'	=>  $produto['NRSEQVENDA']
			);

			$infoProduto = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::SQL_GET_PRODUCT_INFO, $params);
			$produtos[$key]['NMPRODUTO'] = $infoProduto['NMPRODUTO'];
			$produtos[$key]['CDARVPROD'] = $infoProduto['CDARVPROD'];
			$produtos[$key]['CDBARPRODUTO'] = $infoProduto['CDBARPRODUTO'];

			$TXPRODPED = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::SQL_GET_TXPRODPED, $params);
			if (!empty($TXPRODPED['TXPRODPED'])){
				$OBSPROD = explode(';', $TXPRODPED['TXPRODPED']);
			} 
			else $OBSPROD = array();
			$produtos[$key]['OBSPROD'] = $OBSPROD;
			
			if($produto['CDCAIXA'] == ''){
				$infoComanda = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::SQL_GET_CMDINFO, $params);
				$produtos[$key]['CDCAIXA'] = $infoComanda['CDCAIXA'];
				$produtos[$key]['NMCAIXA'] = $infoComanda['NMCAIXA'];
				$produtos[$key]['CDLOJA'] = $infoComanda['CDLOJA'];
				$produtos[$key]['NMLOJA'] = $infoComanda['NMLOJA'];
			}else{
				$infoVenda = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::SQL_GET_VNDINFO, $params);
				$produtos[$key]['CDCAIXA'] = $infoVenda['CDCAIXA'];
				$produtos[$key]['NMCAIXA'] = $infoVenda['NMCAIXA'];
				$produtos[$key]['CDLOJA'] = $infoVenda['CDLOJA'];
				$produtos[$key]['NMLOJA'] = $infoVenda['NMLOJA'];
			}
			
			$CDSENHAPED = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::SQL_GET_SENHAPED, $params);
			$produtos[$key]['CDSENHAPED'] = $CDSENHAPED['CDSENHAPED'];

		}
		return $produtos;
	}

}