function SelectMultipleController($scope, $rootScope, templateManager, ScreenService, ZhUtil, SelectService, ZHPromise, metaDataFactory, EventChildPropagation) {

	var _openField = function(field) {

		var defer = ZHPromise.defer();
		var actions = [{
			"label": ScreenService.i18n("Cancel"),
			"showAsAction": "back",
			"isVisible": true,
			"showOnForm": true,
			"showOnList": true,
			"hideIcon": true,
			"events": [{
				"name": "ActionEvent",
				"code": "ScreenService.closePopup(); args.owner.widget.afterClosePopup(args.owner.widget.selectField);",
				"id": 9998
			}]
		}, {
			"label": ScreenService.i18n("Ok"),
			"showAsAction": "never",
			"isVisible": true,
			"showOnForm": true,
			"showOnList": true,
			"events": [{
				"name": "ActionEvent",
				"code": "args.owner.widget.setSelected(args.owner.widget.dataSource.getCheckedRows(), args.owner.widget.selectField); args.owner.widget.afterClosePopup(args.owner.widget.selectField);",
				"id": 9999
			}]
		}];

		if (field.showAddButtonAs == 'action') {
			actions.push({
				"icon": "plus",
				"showAsAction": "always|edit|checked_rows|view",
				"isVisible": true,
				"showOnForm": true,
				"showOnList": true,
				"events": [{
					"name": "ActionEvent",
					"code": "args.owner.widget.onAdd()",
					"id": 9999
				}]
			});
		}

		innerSearch("", field);

		var template;
		if (!field.showSelectAs) {
			template = "widget/grid.html";
		} else if (field.showSelectAs === "tree") {
			template = "widget/tree/grid.html";
		} else {
			template = "widget/" + field.showSelectAs + ".html";
		}

		var fields;
		if (field.fields && field.fields.length) {
			fields = field.fields;
		} else {
			fields = [{
				"label": field.sourceLabel,
				"name": field.descriptionField,
				"showOnList": true
			}];
		}

		var updateCheckedRows = function() {
			if (field.buildOptionsFromDataSource) {
				field.buildOptionsFromDataSource(field.value());
			}
			
			if (Util.isArray(field.widget.currentRow[field.name])) {
				var rowsToRemove = [];
				if (Util.isArray(field.valueFields)) {
					rowsToRemove = field.dataSource.getCheckedRows().filter(function(row) {
						return field.widget.currentRow[field.name].every(function(value) {
							return field.valueFields.some(function(key) {
								return value[key] !== row[key];
							});
						});
					});
				} else {
					rowsToRemove = field.dataSource.getCheckedRows().filter(function(row) {
						return field.widget.currentRow[field.name].indexOf(row[field.valueField]) === -1;
					});
				}
				field.selectWidget.dataSource.removeCheckedRows(rowsToRemove);
			} else {
				field.selectWidget.dataSource.clearCheckedRows();
			}
			defer.resolve();
		};

		var isToReload = !!(field.selectWidget && Util.objectHasAnyValue(field.selectWidget.searchCriteria));
		if (!field.selectWidget) {
			var widgetEvents = [{
				"name": "WidgetAfterMoveRow",
				"code": "args.owner.toggle(args.row, args.owner);",
				"id": 9999
			}];
			field.selectWidget = metaDataFactory.widgetFactory({
				"id": 9999,
				"name": "select_multiple_widget",
				"label": ScreenService.i18n(field.label),
				"template": template,
				"container": field.widget.container,
				"isVisible": true,
				"popupNoBlock": false,
				"showDescriptionOrder": false,
				"floatingControl": {
					"openSearch": !Util.isMobile(),
					"defaultSearchField": field.defaultSearchField || field.descriptionField,
					"pdfAction": false,
					"xlsAction": false,
					"searchAction": true,
					"customizationAction": {
						"filter": false,
						"layout": false,
						"view": false
					}
				},

				"parentProperty": field.parentProperty,
				"primaryProperty": field.primaryProperty,
				"expandingProperty": field.expandingProperty,
				"breadcrumbProperty": field.breadcrumbProperty,
				"selectCascade": field.selectCascade,
				"childrenDependency": field.childrenDependency,

				"order": field.order,
				"reverse": field.reverse,
				"bringGroupsCollapsed": field.bringGroupsCollapsed,
				"selectField": field,
				"showCheckbox": true,
				"onAdd": field.onAdd,
				"showAddButtonAs": field.showAddButtonAs,
				"addButtonText": field.addButtonText,
				"noMatchText": field.noMatchText,
				"searchNoMatchText": field.searchNoMatchText,
				"itemsPerPage": field.itemsPerPage,
				"searchWithPagination": Util.isUndefinedOrNull(field.searchWithPagination) ? true : field.searchWithPagination,
				"hasPagination": Util.isUndefinedOrNull(field.hasPagination) ? true : field.hasPagination,
				"onDemand": field.onDemand,
				"searchCriteria": field.searchCriteria,

				"fields": fields,
				"dataSource": field.dataSource,
				"dataSourceFilter": field.dataSourceFilter,
				"events": EventChildPropagation.propagate(widgetEvents, field),
				"actions": actions
			});
			field.selectWidget.setSelected = setSelected;
			field.selectWidget.afterClosePopup = afterClosePopup;
			isToReload = !!(field.dataSource && field.dataSource.lazyLoad);
		}

		if (isToReload) {
			field.selectWidget.searchCriteria = {};
			field.selectWidget.searchList = "";
			field.reload().then(updateCheckedRows, updateCheckedRows);
		} else {
			updateCheckedRows();
		}

		ScreenService.openPopup(field.selectWidget);
		return defer.promise;
	};

	$scope.openField = function(field) {
		var e = false;
		if (!field.isReadOnly()) {
			if (field.beforeSelectOpen) {
				e = field.beforeSelectOpen();
				if (e.isDefaultPrevented()) {
					return;
				}
			}
			// remove focus from element
			field.element.blur();
			window.getSelection().removeAllRanges(); //bug webkit

			_openField(field).then(function() {
				if (field.afterSelectOpen) {
					field.afterSelectOpen();
				}
			});
		}
	};

	function innerSearch(selectListSearch, field) {
		if (Util.isArray(field.filterDefinitions) && field.filterDefinitions.length) {
			var dataSource = field.dataSource;
			var filter = ZhUtil.buildFiltersInField(field, selectListSearch);
			dataSource.filter(filter, function(data) {
				dataSource.data = data;
				templateManager.updateTemplate();
			});
		}
	}

	var setChanged = function(field) {
		field.widget.currentRow.changed = true;
	};

	function setSelected(rows, field) {
		ScreenService.closePopup();

		(field.outData ? changeByOut : changeByName)(field, rows);
		setChanged(field);

		if (field._afterSelected) {
			field._afterSelected(row);
		}

		if (field.change) field.change();
	}

	function afterClosePopup(field) {
		if (field.element) {
			field.element.focus();
		}
	}

	function changeByName(field, rows) {
		field.widget.currentRow[field.name] = [];
		if (Util.isArray(field.valueFields)) {
			rows.forEach(function(item) {
				field.widget.currentRow[field.name].push(field.valueFields.reduce(function(value, key) {
					value[key] = item[key];
					return value;
				}, {}));
			});
		} else {
			rows.forEach(function(item) {
				field.widget.currentRow[field.name].push(item[field.valueField]);
			});
		}
	}

	function changeByOut(field, rows) {
		field.widget.currentRow[field.name] = [];
		if (Util.isArray(field.valueFields)) {
			rows.forEach(function(item) {
				field.widget.currentRow[field.name].push(field.valueFields.reduce(function(value, key) {
					value[key] = item[key];
					return value;
				}, {}));
			});
		} else {
			rows.forEach(function(item) {
				field.widget.currentRow[field.name].push(item[field.valueField]);
			});
		}
		
		if (field.widget.currentRow) {
			var outData = field.outData || {};
			prepareCurrentRow(outData, field, rows);
			var keys = Object.keys(outData);
			rows.forEach(function(row) {
				keys.forEach(function(key) {
					if (Array.isArray(outData[key])) {
						outData[key].forEach(function(valueKey) {
							field.widget.currentRow[valueKey].push(row[key]);
						});
					} else {
						field.widget.currentRow[outData[key]].push(row[key]);
					}
				});
			});
		}
	}

	function prepareCurrentRow(outData, field, rows) {
		var keys = Object.keys(outData);
		keys.forEach(function(key) {
			if (Array.isArray(outData[key])) {
				outData[key].forEach(function(valueKey) {
					field.widget.currentRow[valueKey] = [];
				});
			} else {
				field.widget.currentRow[outData[key]] = [];
			}
		});
	}

	$scope.maintainWindow = function(field) {
		if (field.maintainWindow) {
			var maintainWindowFilter = SelectService.buildMaintainFilter(field);
			ScreenService.openWindow(field.maintainWindow, maintainWindowFilter);
		}
	};

	$scope.$watchCollection('field.dataSourceFilter', propagateFieldDataSourceFilterToSelectWidget);
	function propagateFieldDataSourceFilterToSelectWidget(fieldDataSourceFilter) {
		if ($scope.field.selectWidget) {
			$scope.field.selectWidget.dataSourceFilter = fieldDataSourceFilter;
		}
	}

	this.hasSelectedRows = function hasSelectedRows(field) {
		var selectedRows = field.value();
		return selectedRows && selectedRows.length > 0;
	};
}