var ZeedhiDirectives = angular.module("ZeedhiDirectives", []);

ZeedhiDirectives.directive('ngInclude', function(ZhLogger) {
	return {
		link: function($scope, element, attrs, controller) {
			if (ZhLogger.level === Logger.DEBUG) {
				// adds the src of the template included
				element.before('<!-- INCLUDE SRC = ' + $scope.$eval(element.attr('src') || element.attr('ng-include') || element.attr('data-ng-include')) +
					' -->');
			}
		}
	};
});

ZeedhiDirectives.directive('zhMask', function(maskEngine, templateManager) {
	return {
		require: '?ngModel',
		priority: 1,
		compile: function(tElement, tAttrs, transclude) {
			return {
				pre: function preLink($scope, element, iAttrs, controller) {
					var maskMethod = false;
					var component = $scope[iAttrs.zhMaskComponent || "field"];
					if (component.mask) {
						var mappedMasks = maskEngine.getMappedMasks();
						var params = component.mask.params || {};
						maskMethod = component.mask.type;
						controller.$formatters.unshift(function(value) {
							var maskFormatMethod = mappedMasks[maskMethod].format;
							var result = false;
							if (value && component.mask && maskFormatMethod) {
								result = maskFormatMethod.apply(maskEngine, [value, params]);
							} else {
								result = value;
							}
							return result;
						});
						
						if (maskMethod) {
							if (params.cleanMaskOnModel) {
								controller.$parsers.push(function (value) {
									//Angular treats undefined as error in parsers
									var parsedValue = Util.isUndefined(value)? null : value;
									var maskParserMethod = mappedMasks[maskMethod].parser;
									if (value && maskParserMethod) {
										parsedValue = maskParserMethod.apply(maskEngine, [value, params]);
									}
									
									return parsedValue;
								});
							}
							
							var callBack = function(value) {
								controller.$setViewValue(value);
								templateManager.updateTemplate();
							};
							var funcMask = mappedMasks[maskMethod].mask;
							funcMask(element, controller.$viewValue, params, callBack);
						} else {
							controller.$viewValue = element.val();
						}
					}
				}
			};
		}
	};
});

ZeedhiDirectives.directive("zhAcl", function(templateManager, aclEngine) {
	return {
		priority: 1001,
		link: function($scope, element, attrs) {

			var aclApply = function() {
				var elementType = attrs.zhAcl;
				if (templateManager.project && templateManager.project.aclEnable) {
					if (elementType === "menu") {
						aclEngine.processMenuAcl(templateManager.project.groupMenu, null, templateManager.aclRules, templateManager.containers, templateManager.userId);
					} else if (elementType !== "menu" && templateManager.container.aclEnable) {
						aclEngine.processAcl(templateManager.container, null, templateManager.aclRules, templateManager.userId);
					}
				}
			};
			aclApply();

			$scope.$watch(angular.bind(templateManager.project, function(a) {
				aclApply();
			}));
		}
	};
});

ZeedhiDirectives.directive('zhStyle', function(ZhUtil, ApplicationContext, ZhLogger) {
	return {
		link: function($scope, element, attrs, controller) {
			var property = attrs.zhStyle || 'row';
			var component = attrs.zhStyleComponent || 'field';

			function evaluateCondition(propertyValue, condition) {
				//@todo: Somebody doesn't like this
				var result = false;
				ApplicationContext.registerInstance('row', propertyValue || {});

				try {
					result = ZhUtil.evaluate("@" + condition + "@");
				} catch (e) {
					ZhLogger.error("Error while evaluating ZhStyle condition function", e);
				}

				ApplicationContext.removeInstance('row');
				return result;
			}

			function callFunction(functionStr, propertyValue) {
				//@todo: Somebody doesn't like this
				var func;
				ApplicationContext.registerInstance('row', propertyValue);
				ApplicationContext.registerInstance('element', element);

				try {
					result = ZhUtil.evaluate(functionStr);
				} catch (e) {
					ZhLogger.error("Error while evaluating ZhStyle styling function", e);
				}

				ApplicationContext.removeInstance('row');
				ApplicationContext.removeInstance('element');
			}

			function customizeByCss(condition, cssClass) {
				if (condition) {
					element.addClass(cssClass);
				} else {
					element.removeClass(cssClass);
				}
			}

			function customizeByFunction(condition, functionStr, propertyValue) {
				if (condition) {
					callFunction(functionStr, propertyValue);
				}
			}


			$scope.$watchCollection(property, function(propertyValue) {

				var customStyles = [];

				if ($scope[component] && $scope[component].styles) {
					customStyles = $scope[component].styles;
				} else if ($scope.widget && $scope.widget.styles) {
					customStyles = $scope.widget.styles;
				}

				customStyles.forEach(function(customization) {
					var conditionResult = evaluateCondition(propertyValue, customization.condition);

					// Simple way, add a CSS class to the element
					if (customization.cssClass) {
						customizeByCss(conditionResult, customization.cssClass);
					}

					// Advanced way, execute a function passing the element and the row as parameters
					if (customization.stylingFunction) {
						customizeByFunction(conditionResult, customization.stylingFunction, propertyValue);
					}

				});

			});

		}
	};
});

ZeedhiDirectives.directive('zhLoader', [function() {

	var timeout = 0;

	function debounceSplash() {
		clearTimeout(timeout);
		timeout = setTimeout(removeClass, 500);
	}

	function removeClass() {
		$("#splash").fadeOut("slow", function() {
			$(this).removeClass('splash');
		});
		timeout = -1;
	}

	return {
		link: function($scope, element, attrs) {
			var started = 0;
			$scope.$on('request.stop', function() {
				if (started > 0) {
					started--;
				}
				if (element.is(':visible') && started === 0) {
					element.hide();
				}
				if (timeout >= 0) {
					debounceSplash();
				}
			});
			$scope.$on('request.start', function() {
				started++;
				if (!element.is(':visible')) {
					element.show();
				}
			});
		}
	};
}]);

ZeedhiDirectives.directive('zhAlert', ['ScreenService', '$rootScope', '$timeout', function(ScreenService, $rootScope, $timeout) {
	return {
		link: function($scope, $element, attrs) {
			$element.hide();
			var hide = function() {
				$element.hide();
				$rootScope.alertShown = false;
			};

			$scope.$on('showCustomDialog', function(e, data) {
				data = data || {};
				$rootScope.alertShown = true;

				$scope.alertText = data.message;
				$scope.alertIcon = data.icon;
				$scope.buttons = data.actions;
				$scope.horizontal = false;

				$element.find('.zh-alert-scroll').nonbounce();
				$element.show();

				$scope.alertButtonClick = function(e, button) {
					hide();
					if (button.code && Util.isFunction(button.code)) {
						button.code(e, data.deferred);
					}
				};

				$timeout(function() {
					$scope.horizontal = (data.actions.length == 2 && $element.find('button span').width() < $element.find('button').width() / 2);
				});
			});
		}
	};
}]);

ZeedhiDirectives.directive('zhAbout', function(ScreenService, $rootScope, templateManager, $timeout) {
	return {
		link: function($scope, $element, attrs) {
			$element.hide();
			var hide = function() {
				$element.hide();
				$rootScope.showAbout = false;
			};

			$scope.$on('showAbout', function(e) {
				$rootScope.showAbout = true;
				$element.show();

				$timeout(function() {
					var versionDataDiv = $element.find('.version-data');
					versionDataDiv.css('top', versionDataDiv.position().top);
					versionDataDiv.css('bottom', $element.find('.powered-by').height() + 10);
				});
			});

			$scope.$on('hideAbout', hide);

			$scope.closeAbout = hide;
		}
	};
});

ZeedhiDirectives.directive('zhHideEmpty', function() {
	return function($scope, $element, $attrs) {
		var empty = true;

		var children = $element.children();
		for (var i = children.length - 1; i >= 0; i--) {
			if (children[i].is(":visible")) {
				empty = false;
				break;
			}
		}
		if (empty) {
			$element.addClass("zh-hide");
		} else {
			$element.removeClass("zh-hide");
		}
	};
});

ZeedhiDirectives.directive('zhInnerTemplate', function() {
	return {
		scope: true,

		restrict: 'A',

		postLink: function($scope, $element, $attrs) {
			var innerWidget = $scope.widget.widgets[0];
			$scope.widget = innerWidget;
			$scope.bootstraped = true;
		}
	};
});

ZeedhiDirectives.directive("zhTabWrapper", function(templateManager, $timeout) {

	return {
		restrict: 'C',
		scope: {
			tabWidget: '=',
			tabWidgets: '='
		},
		link: function(scope, elm, attrs) {

			var element = elm[0];

			var scroll;
			$timeout(function() {
				scroll = new IScroll(element, {
					eventPassthrough: true,
					scrollX: true,
					scrollY: false,
					preventDefault: false
				});
				templateManager.scrollers = templateManager.scrollers || [];
				templateManager.scrollers.push(scroll);
			}, 500); // prevent calculation of scrolling before swipe resizing

			var object = {
				firstRefresh: 3000,
				regularRefresh: 1000,
				interval: 100,
				clearScheduler: function() {
					clearTimeout(object.timer);
					object.currentTime = 0;
					// when it's the second time running decreases the refresh rate
					// from object.firstRefresh to object.regularRefresh
					object.refresh = object.firstRun ? object.regularRefresh : object.firstRefresh;
				},
				startUpdate: function() {
					// if the tabWidget or tabWidgets cease to exist, do not set schedule.
					// this happens when a tabbedWidget is inside a tabbed container
					// and the container changes tabs.
					object.clearScheduler();
					if (scope.tabWidget && scope.tabWidgets) {
						object.setScheduler();
					}
				},
				setScheduler: function() {
					// checks if it should refresh again, and if so, increments the timer.
					// otherwise, time's reset and no other event is scheduled.
					// redundant verification for the existance of the tabWidget and tabWidgets objects.
					if ((object.currentTime < object.refresh) && (scope.tabWidget && scope.tabWidgets)) {
						object.udpateElements();
						object.timer = setTimeout(object.setScheduler, object.interval);
						object.currentTime += object.interval;
					} else {
						object.firstRun = true;
					}
				},
				udpateElements: function() {
					// does simple math to decide the size of the line and its position.
					object.elementDiv = object.elementDiv || $($(element).find('div')[0]);
					object.elementLine = object.elementLine || object.elementDiv.find('.zh-tab-line');
					object.elementCell = object.elementDiv.find('.zh-tab-cell');

					var index = scope.tabWidgets.filter(function(widget) {
						return widget.isVisible && widget.aclIsVisible && widget.showAsTab;
					}).indexOf(scope.tabWidget);
					var elementWidth = $(object.elementCell[index]).width(),
						priorWidth = object.getPriorWidth(index);

					object.elementLine.width(elementWidth);
					object.elementLine.css('left', priorWidth);
				},
				getPriorWidth: function(index) {
					// gets the index of a tab and returns the width of its combined predecessors
					var priorWidth = 0;
					for (i = 0; i < index; i++) {
						priorWidth += $(object.elementCell[i]).width();
					}
					return index ? priorWidth + (index * 15) : priorWidth;
				}
			};

			['tabWidget', 'tabWidgets'].forEach(function(each) {
				scope.$watch(each, object.startUpdate);
			});
		}
	};

});

ZeedhiDirectives.directive('zhCenterTop', function() {
	return {
		link: function($scope, element, iAttrs) {
			element.parent().addClass('zh-center-top-parent');
		}
	};
});

ZeedhiDirectives.directive('zhKanbanTask', ["$timeout", function($timeout) {
	return {
		restrict: 'A',
		link: function($scope, element) {
			$scope.taskClick = function() {
				if ($scope.widget.onTaskClick) {
					$scope.widget.onTaskClick({
						item: $scope.item
					});
				}
			};
		}
	};
}]);

ZeedhiDirectives.directive('zhTaskMatrix', ["$timeout", function($timeout) {
	return {
		restrict: 'A',
		link: function($scope, element) {

			// Sets maximum x-axis and y-axis values
			var maxY = $scope.widget.maxY || 10;
			var maxX = $scope.widget.maxX || 10;

			var clickedElems = [];

			// Creates a DOM reference matrix
			var matrix = new Array(maxX);
			for (var i = 0; i < maxX; i++) {
				matrix[i] = new Array(maxY);
				for (var j = 0; j < maxY; j++) {
					matrix[i][j] = [];
				}
			}

			function moveRow(evt) {
				var row;
				var xValue = $(evt.target).attr("xValue");
				var yValue = $(evt.target).attr("yValue");

				matrix[xValue][yValue].forEach(function(item) {
					if (item.elem[0] == evt.target) {
						$scope.widget.setCurrentRow(item.row);
						dismissSelection();
					}
				});
			}

			function dismissSelection() {
				element.find(".cover").css({
					"visibility": "hidden",
					"opacity": "0"
				});
				clickedElems.forEach(function(item) {
					item.elem.css({
						"z-index": "0",
						"margin-top": item.marginTop,
						"margin-left": item.marginLeft
					});
					item.elem.unbind("click");
					item.elem.click(selectData);
				});
				clickedElems = [];
			}

			function selectData(evt) {
				var elem = $(evt.target);
				var posx = elem.attr("xValue");
				var posy = elem.attr("yValue");

				if (matrix[posx][posy].length > 1) {
					// Shows cover
					element.find(".cover").css({
						"visibility": "visible",
						"opacity": "0.75"
					});

					// Sets clicked group
					clickedElems = matrix[posx][posy];

					// Styles the elements
					matrix[posx][posy].forEach(function(item, i) {
						item.marginTop = item.elem.css("margin-top");
						item.marginLeft = item.elem.css("margin-left");

						item.elem.css({
							"z-index": "1",
							"margin-top": "-18px",
							"margin-left": Math.pow(-1, i + 1) * Math.floor((i + 1) / 2) * (parseInt(item.elem.width()) + 10) - 12
						});

						item.elem.unbind("click");
						item.elem.click(moveRow);
					});
				} else {
					moveRow(evt);
				}
			}

			function loadData() {
				var wrapper = element.find(".data-squares").empty();
				$scope.widget.dataSource.data.forEach(function(row, index) {

					// Initial values
					var xValue = row[$scope.widget.xField];
					var yValue = row[$scope.widget.yField];
					var color = row[$scope.widget.colorField];
					var displacement = matrix[xValue][yValue].length * 4;

					// Creates element and puts it's reference in the matrix
					var elem = angular.element("<div></div>");
					matrix[xValue][yValue].push({
						"elem": elem,
						"row": row,
						"index": index
					});

					var valueFieldPropery = Util.isArray($scope.widget.valueField) ? $scope.widget.valueField[0] : $scope.widget.valueField;
					elem.html(row[valueFieldPropery]);

					elem.attr("xValue", xValue);
					elem.attr("yValue", yValue);

					// Sets element styles
					elem.addClass("task");
					elem.css({
						"top": (100 - ((100 / maxY) * yValue)) + "%",
						"left": ((100 / maxX) * xValue) + "%",
						"margin-top": displacement - 18 + "px",
						"margin-left": "-18px",
						"backgroundColor": color || "#e2a53b"
					});

					// Binds a click function to elements
					$(elem).click(selectData);
					element.find(".cover").click(dismissSelection);

					// Appends element to DOM
					wrapper.append(elem);
				});
			}

			function generateScale() {
				var xAxis = element.find(".x-axis-scale").empty();
				var elem;
				for (var j = 1; j < maxX; j++) {
					elem = angular.element("<span></span>");
					elem.css({
						"left": j * (100 / maxX) + "%"
					});
					elem.html(j);
					xAxis.append(elem);
				}

				var yAxis = element.find(".y-axis-scale").empty();
				for (var k = 1; k < maxY; k++) {
					elem = angular.element("<span></span>");
					elem.css({
						"bottom": k * (100 / maxY) + "%"
					});
					elem.html(k);
					yAxis.append(elem);
				}
			}

			$scope.$watch("widget.dataSource.data", function() {
				generateScale();
				loadData();
			});
		}
	};
}]);

ZeedhiDirectives.directive('zhStripe', ["$timeout", function($timeout) {
	return {
		restrict: 'A',
		link: function($scope, element) {

			// fix stripe rendering due to async loading of templates
			// if find first sibling container has been rendered before the stripe, adjusts its top.
			if (!element.parent().hasClass('stripe-footer') && parseInt(element.parent().next().find('.container').eq(0).css('top')) === 0) {
				element.parent().next().find('.container').eq(0).css('top', element.outerHeight());
			}

			if (element.parent().hasClass('stripe-footer') && parseInt(element.parent().prev('.container').eq(0).css('bottom')) === 0) {
				element.parent().prev('.container').eq(0).css('bottom', element.outerHeight());
			}

			$scope.getAdjustedMaxWidth = function() {
				var stripeRightSide = element.find('.right-side').width();
				var elementWidth = element.width();
				return {
					"position": "relative",
					"max-width": elementWidth - stripeRightSide + "px"
				};
			};
		}
	};
}]);

ZeedhiDirectives.directive('zhFullWidthFont', ["$timeout", function($timeout) {
	return {
		restrict: 'A',
		link: function($scope, element) {

			var parent = element.parent();

			function scaleFont() {

				var opts = {};
				if (element.attr("zh-full-width-font")) {
					opts = JSON.parse(element.attr("zh-full-width-font"));
				}

				var maxWidth = parent.width();
				var maxHeight = opts.maxHeight || 16;
				var span = element.find(".full-width");

				var fontSize = maxHeight;
				var textWidth;
				var textHeight;
				do {
					span.css('font-size', fontSize);
					textHeight = span.height();
					textWidth = span.width();
					fontSize = fontSize - 1;
				} while ((textHeight > maxHeight || textWidth > maxWidth) && fontSize > 3);
			}

			$timeout(function() {
				scaleFont();
			}, 250);

			parent.on("resize.item", Util.buildDebounceMethod(function() {
				scaleFont();
			}, 100));

		}
	};
}]);


ZeedhiDirectives.directive('zhNotificationShortcut', ["$timeout", function($timeout) {
	return {
		restrict: 'A',
		link: function($scope, element) {

			var parent = element.parent();
			var fontHeight = 16;

			function scaleIcon() {
				var size = Math.min(parent.height(), parent.width()) - 20 - 10 - fontHeight; // 20px of padding and 10px of gap
				element.find(".shortcut-icon").css({
					"transform": "translate(-50%, -50%) scale(" + (size / 40) + ")"
				});
			}

			function setIconFill() {
				var color = parent.css("background-color");
				element.find("svg *").css({
					"fill": color
				});
			}

			$timeout(function() {
				scaleIcon();
				setIconFill();
			}, 250);

			parent.on("resize.item", Util.buildDebounceMethod(function() {
				fontHeight = parseInt(element.find(".full-width").css("font-size"));
				scaleIcon();
			}, 110));

		}
	};
}]);

ZeedhiDirectives.directive('zhFilledIconNotification', ["$timeout", function($timeout) {
	return {
		restrict: 'A',
		link: function($scope, element) {

			var parent = element.parent();

			function scaleIcon() {
				var wrapper = element.find('.wrapper');
				var size = Math.min(wrapper.height(), 0.6 * wrapper.width()); // 20px of padding and 10px of gap
				element.find(".shortcut-icon svg").css({
					"transform": "translate(-50%, -50%) scale(" + (1 + size / 40) + ")",
					"-webkit-transform": "translate(-50%, -50%) scale(" + (1 + size / 40) + ")"
				});
			}

			function scaleFont() {
				var notification = element.find('.shortcut-notification');

				var maxWidth = notification.width();
				var span = element.find(".full-width");

				var fontSize = notification.height();
				var textWidth;
				do {
					span.css('font-size', fontSize);
					textWidth = span.width();
					fontSize = fontSize - 1;
				} while (textWidth > maxWidth && fontSize > 3);

				span.css('line-height', notification.height() + 'px');
			}

			$timeout(function() {
				scaleIcon();
				scaleFont();
			}, 250);

			parent.on("resize.item", Util.buildDebounceMethod(function() {
				scaleIcon();
				scaleFont();
			}, 100));

		}
	};
}]);

ZeedhiDirectives.directive('zhPizza', ["$timeout", "maskEngine", function($timeout, maskEngine) {
	return {
		restrict: 'A',
		link: function($scope, $element) {

			var widget = $scope.widget;
			var container, canvas, ctx, chart, legend;

			container = $element;
			canvas = $element.find("canvas");

			var legendTemplate =
				"<ul class=\"pie-legend\">" +
				"<% for (var i=0; i<segments.length; i++) { %>" +
				"<li>" +
				"<span class=\"slice-color\" style=\"background-color: <%= segments[i].fillColor %>\"></span>" +
				"<span class=\"slice-label\">" +
				"<%= segments[i].label %>" +
				"</span>" +
				"</li>" +
				"<% } %>" +
				"</ul>}";

			var colors = widget.colors || ['#f1c40f', '#f39c12', '#e67e22', '#d35400', '#e74c3c', '#c0392b', '#9b59b6', '#8e44ad', '#2ecc71', '#27ae60',
				'#1abc9c', '#16a085', '#3498db', '#2980b9', '#34495e', '#2c3e50', '#95a5a6', '#7f8c8d', '#bdc3c7', '#ecf0f1'
			];

			function getWidgetFormat(widget) {
				if (widget.mask) {
					return widget.mask;
				} else if (!widget.mask && widget.valueFormat) {
					return {
						type: widget.valueFormat
					};
				}
				return {};
			}

			var pieOpts = {
				legendTemplate: legendTemplate,
				tooltipTemplate: function(activePoint) {
					return activePoint.label + ": " + Util.applyMask(activePoint.value, getWidgetFormat(widget), maskEngine, ZhUtil);
				}
			};

			function buildPizzaChart() {
				/* Chart building */
				var pieData = [];
				var valueFieldPropery = Util.isArray(widget.valueField) ? widget.valueField[0] : widget.valueField;
				widget.dataSource.data.forEach(function(item, index) {
					pieData.push({
						value: item[valueFieldPropery] ? item[valueFieldPropery].toString().replace(/,/g, ".") : 0,
						label: item[widget.labelField],
						color: item[widget.colorField] || colors[index % colors.length],
						highlight: item[widget.colorField] || colors[index]
					});
				});

				var cwidth = (container.parents(".zh-dashboard-chart").length && !Util.isPhone()) ? 0.65 * container.width() : container.width();
				var cheight = container.height();

				canvas.attr({
					"width": cwidth - 20,
					"height": cheight - 140
				});

				canvas.css({
					"width": cwidth - 20,
					"height": cheight - 140
				});

				ctx = canvas[0].getContext("2d");
				chart = new Chart(ctx).Pie(pieData, pieOpts);

				canvas[0].onclick = function(evt) {
					var activePoints = chart.getSegmentsAtEvent(evt);
					if (activePoints && activePoints.length > 0) {
						widget.setCurrentRow(widget.dataSource.data[chart.segments.indexOf(activePoints[0])]);
					}
				};
			}

			function buildPizzaLegend() {
				legend = $(chart.generateLegend());
				container.find(".pie-legend").remove();
				container.append(legend);

				var helpers = Chart.helpers;

				helpers.each(legend.children(), function(legendChild, index) {

					helpers.removeEvent(legendChild, 'mouseover');
					helpers.removeEvent(legendChild, 'mouseout');

					helpers.addEvent(legendChild, 'mouseover', function() {
						var activeSegment = chart.segments[index];
						activeSegment.save();
						activeSegment.fillColor = activeSegment.highlightColor;
						chart.showTooltip([activeSegment]);
						activeSegment.restore();
					});

					helpers.addEvent(legend, 'mouseout', function() {
						chart.draw();
					});

					legendChild.onclick = function(evt) {
						widget.setCurrentRow(widget.dataSource.data[index]);
					};

				});
			}

			function drawPizza() {
				if (chart) {
					chart.destroy();
				}
				buildPizzaChart();
				buildPizzaLegend();
			}

			widget.redraw = function() {
				drawPizza();
			};

			$(window).on('resize.' + widget.name, function() {
				if (canvas.is(':visible')) {
					drawPizza();
				} else {
					$(window).off('resize.' + widget.name);
				}
			});


			$scope.$watch("widget.dataSource.data", function() {
				$timeout(function() {
					drawPizza();
				}, 200);
			});
		}
	};
}]);