describe("Container", function () {
	var container = {};
	var baseContainer = {
		"widgets": [{
			"id": 1,
			"name": "first",
			"fields": [],
			"active": true,
			"actions": [{
				"showAsAction": "checked_rows"
			}, {

				"showAsAction": "view"
			}, {
				"showAsAction": "edit|view"
			}, {
				"showAsAction": "edit"
			}, {
				"showAsAction": "edit_edit"
			}]
		}, {
			"id": 2,
			"name": "last",
			"fields": [],
			"active": true,
			"widgets": [{
				"id": 2.1,
				"name": "last.first",
				"fields": [],
				"active": true
			}],
			"actions": [{
				"showAsAction": "checked_rows"
			}, {

				"showAsAction": "view"
			}, {
				"showAsAction": "edit|view"
			}, {
				"showAsAction": "edit"
			}]
		},]
	};
	var localStorageEngine, requestDataSourceEngine, SQLStorageStrategy, restEngine = null;
	var eventEngine = new EventEngine(new EventFactory(this, new MethodBuilder({}, {}), {}, {
		i18n: function (a) {
			return a;
		}
	}));
	var aclEngine = new AclEngine();
	var screenService = {
		isPopupOpen: function () {
			return false;
		}
	};
	var metaDataFactory = new MetaDataFactory(localStorageEngine, requestDataSourceEngine, SQLStorageStrategy, restEngine, eventEngine, screenService, aclEngine, {
		project: {}
	}, {}, new ZHPromise(Q));
	beforeEach(function () {
		container = metaDataFactory.containerFactory(baseContainer);
		container.eachWidget(function (widget) {
			widget.active = true;
		});
	});
	it("Container should be created correct", function () {
		expect(container.getActionsByVisibility).not.toEqual(undefined);
	});

	it("Container getActions should return all actions", function () {
		expect(container.getActions().length).toEqual(9);
	});

	it("Container getActions should return actions with visibility using | as a separator", function () {
		expect(container.getActions("edit").length).toEqual(4);
	});

	it("Container getActions should return actions with visibility using with arry parameter", function () {
		expect(container.getActions(["edit", "view"]).length).toEqual(6);
	});

	it("Container getActions should return actions with visibility using with arry parameter", function () {
		expect(container.getActions(["edit", "view"]).length).toEqual(6);
	});

	it("Container getActions should return actions with visibility using with arry parameter only of the active Widget", function () {
		container.widgets[1].active = false;
		expect(container.getActions(["edit", "view"]).length).toEqual(3);
	});

	it("Container getActions should return actions with visibility only active widgets", function () {
		container.widgets[1].active = false;
		expect(container.getActions("edit").length).toEqual(2);
	});

	it("Container getActionsByVisibility should return actions with visibility and visibility_$mode", function () {
		container.widgets[1].active = false;
		container.mode = "edit";
		expect(container.getActionsByVisibility("edit").length).toEqual(3);
	});

	it("Container getActionsByVisibility should return actions with visibility and visibility_$mode", function () {
		container.widgets[1].active = true;
		container.mode = "edit";
		expect(container.getActionsByVisibility("edit").length).toEqual(5);
	});

	it("Container eachWidget should apply function to all widgets and his children()", function () {
		container.eachWidget(function (widget) {
			widget.visited = true;
		});
		expect(container.widgets[0].visited).toEqual(true);
		expect(container.widgets[1].visited).toEqual(true);
		expect(container.widgets[1].widgets[0].visited).toEqual(true);
	});

	it("Method getWidget should be recursive when use name", function () {
		expect(container.getWidget('last.first').id).toEqual(2.1);
	});
	it("Method getWidget should return correct widget every time", function () {
		expect(container.getWidget().id).toEqual(1);
		expect(container.getWidget(0).id).toEqual(1);
		expect(container.getWidget(1).id).toEqual(2);
		expect(container.getWidget('first').id).toEqual(1);
		expect(container.getWidget(container.widgets[0]).id).toEqual(1);
	});

	it("Method getActiveWidget should return widget with active=true", function () {
		container.widgets[0].activate();
		expect(container.widgets[0].active).toEqual(true);
		expect(container.widgets[1].active).toEqual(false);

		expect(container.getActiveWidget()).toEqual(container.widgets[0]);
		container.widgets[1].activate();
		expect(container.getActiveWidget()).toEqual(container.widgets[1]);

	});

	it("Method getActiveWidget should return widget with active=true recursively", function () {
		container.widgets[0].widgets = [metaDataFactory.widgetFactory({ container: container, name: "activeWidget" })];
		container.widgets[0].widgets[0].activate();
		expect(container.widgets[0].widgets[0].active).toBeTruthy();
		expect(container.widgets[0].active).toBeFalsy();
		expect(container.widgets[1].active).toBeFalsy();

		expect(container.getActiveWidget()).toEqual(container.widgets[0].widgets[0]);
		container.widgets[1].activate();
		expect(container.getActiveWidget()).toEqual(container.widgets[1]);

	});

	it("Method getActiveWidget should return a factored widget if there is not an active widget", function () {
		var factoredWidget = metaDataFactory.widgetFactory({ container: container, name: "activeWidget" });

		expect(factoredWidget.name).toEqual("activeWidget");
		factoredWidget.activate();
		expect(factoredWidget.active).toBeTruthy();

		expect(container.getActiveWidget()).toEqual(factoredWidget);
	});

	it("Method inViewMode should return true if the container is in View mode", function () {
		container.changeMode(ViewMode.VIEW);
		expect(container.inViewMode()).toBeTruthy();

		container.changeMode(ViewMode.EDIT);
		expect(container.inViewMode()).toBeFalsy();
	});

	it("Method inEditMode should return true if the container is in Edit mode", function () {
		container.changeMode(ViewMode.EDIT);
		expect(container.inEditMode()).toBeTruthy();

		container.changeMode(ViewMode.VIEW);
		expect(container.inEditMode()).toBeFalsy();
	});

	it("Method inListMode should return true if the container is in inListMode mode", function () {
		container.changeMode(ViewMode.LIST);
		expect(container.inListMode()).toBeTruthy();

		container.changeMode(ViewMode.EDIT);
		expect(container.inListMode()).toBeFalsy();
	});

	it("Method countActionsByVisibility count the actions by visibility", function () {
		var desiredVisibility = "always";
		container.actions = [{ isVisible: true, aclIsVisible: true, showAsAction: desiredVisibility, widget: { active: true } }, { isVisible: false, showAsAction: desiredVisibility, widget: {} }];

		expect(container.countActionsByVisibility(desiredVisibility)).toEqual(1);
		container.actions[0].isVisible = false;
		expect(container.countActionsByVisibility(desiredVisibility)).toEqual(0);
	});

	it("Method getActionsTypeToShow should return according to the state", function () {
		container.widgets[0].dataSource = {
			getCheckedRows: function () {
				return [{}, {}];
			},
			updateCheckedRows: function () { }
		};
		container.widgets[0].activate();

		expect(container.getActionsTypeToShow()).toEqual("checked_rows");

		container.widgets[0].dataSource.getCheckedRows = function () { return 0; };

		container.changeMode(ViewMode.EDIT);
		expect(container.getActionsTypeToShow()).toEqual("edit");

		container.changeMode(ViewMode.VIEW);
		expect(container.getActionsTypeToShow()).toEqual("view");

		container.changeMode("ANY OTHER THING");
		expect(container.getActionsTypeToShow()).toEqual("always");
	});

	it("Method getParent should return null", function () {
		expect(container.getParent()).toBeNull();
	});
	
	it("Method inactivateWidget should call deactivate function for fields with widgets", function () {
		var selectFieldMock = {
			selectWidget : {
				deactivate : function() {}
			}
		};
		
		spyOn(selectFieldMock.selectWidget, "deactivate");
		container.widgets[0].fields.push(selectFieldMock);
		
		container.widgets[0].activate();
		
		expect(selectFieldMock.selectWidget.deactivate).toHaveBeenCalled();
	});
});