function OrderActionService($interval, $timeout, environment, NavigatorService, SocketServiceKDS, ConcludeQueue, SoundService, ScreenService) {

    this.startOrderProduction = function (order) {
        var widget = environment.getKDSWidget();
        if (SocketServiceKDS.isOpen() && SocketServiceKDS.isConnected()) {
            var index = NavigatorService.filterOrders(
                widget.dataSource.data
            ).indexOf(order);
            var itemNotCancelled = order.items.find((item) => {
                return item.IDSITITPEDFOS != 'C';
            });
            NRITPEDIDOFOS = itemNotCancelled && itemNotCancelled.NRITPEDIDOFOS;
            SocketServiceKDS.playRequest(order.CDFILIAL, order.NRPEDIDOFOS, NRITPEDIDOFOS, order.CDSETOR);
                widget.dataSource.data.forEach((summaryOrder, idx) => {
                if (summaryOrder.IDSOMRESUMO == 'S' && summaryOrder.NRPEDIDOFOS == order.NRPEDIDOFOS) {
                    SocketServiceKDS.playRequest(summaryOrder.CDFILIAL, summaryOrder.NRPEDIDOFOS, summaryOrder.NRITPEDIDOFOS, summaryOrder.CDSETOR);
                }
            });
            NavigatorService.setOrderAsSelected(index, 'keep');
        } else {
            ScreenService.notificationMessage("Não foi possível iniciar o item, falha de conexão com o servidor.", 'error', 4000);
        }
    };

    this.concludeOrderProduction = function (order) {
        const widget = environment.getKDSWidget();

        if (widget && !widget.completedOrders) {
            // array utilizado para guardar chaves dos pedidos ja removidos da tela
            widget.completedOrders = [];
        }

        let lastSelectedOrder = NavigatorService.filterOrders(widget.dataSource.data).findIndex(of => of.isSelected);

        if (SocketServiceKDS.isOpen() && SocketServiceKDS.isConnected()) {

            let indexOfSelectedOrder = NavigatorService.filterOrders(widget.dataSource.data).indexOf(order);
            let selecteOrderOnDataSource = NavigatorService.filterOrders(widget.dataSource.data)[indexOfSelectedOrder];
            
            if(selecteOrderOnDataSource && order) {
                ConcludeQueue.insert(order, false);
                selecteOrderOnDataSource.isInConclusionQueue = true;

                let key = selecteOrderOnDataSource.CDSETOR + selecteOrderOnDataSource.NRPEDIDOFOS + selecteOrderOnDataSource.NRITPEDIDOFOS;
                widget.completedOrders.push(key); // Armazena a chave do pedido para impedir que ele volte pra tela

                widget.dataSource.data.forEach((summaryOrder, idx) => {
                    if (summaryOrder && (summaryOrder.IDSOMRESUMO == 'S' && summaryOrder.NRPEDIDOFOS == NavigatorService.filterOrders(widget.dataSource.data)[indexOfSelectedOrder].NRPEDIDOFOS)) {
                        ConcludeQueue.insert(summaryOrder, false);
                        summaryOrder.isInConclusionQueue = true;
                    }
                });

                //widget.redrawKDSContainer
                if (!lastSelectedOrder) {
                    let toGo = indexOfSelectedOrder || lastSelectedOrder;
                    toGo = widget.dataSource.data.length == toGo ? toGo -1 : toGo;
                    NavigatorService.setOrderAsSelected(toGo, 'move');
                } else {
                    NavigatorService.setOrderAsSelected(lastSelectedOrder, 'move');
                }
            }
        } else {
            ScreenService.notificationMessage("Sem conexão no momento, tente novamente mais tarde!", 'error', 3000);
        }
    };

    this.expediteOrder = function (order) {
        var widget = environment.getKDSWidget();
        if (SocketServiceKDS.isOpen() && SocketServiceKDS.isConnected()) {
            ConcludeQueue.insert(order, widget.isZoom);
            var indexOfSelectedOrder = NavigatorService.filterOrders(
                widget.dataSource.data
            ).indexOf(order);
            let filteredOrders = NavigatorService.filterOrders(widget.dataSource.data);
            if (filteredOrders[indexOfSelectedOrder]){
                NavigatorService.filterOrders(
                    widget.dataSource.data
                )[indexOfSelectedOrder].isInConclusionQueue = true;
            }
            //widget.redrawKDSContainer
            NavigatorService.setOrderAsSelected(indexOfSelectedOrder, 'move');

        } else {
            ScreenService.notificationMessage("Sem conexão no momento, tente novamente mais tarde!", 'error', 3000);
        }
    };

    this.continueOrder = function (order) {
        order.isPausedByInterval = false;
        order.updateAvailableAction();
        order.startOrderTimer();
    };

    // Controla se as ações disparadas no Front pelo usuário podem ser executadas naquele momento
    // ou se devem aguardar o tempo de delay definido no setTimeout dentro de orderOnClick
    this.isInDelayTime = false;

    this.orderOnClick = function (order, isAutoClick) {
        var widget = environment.getKDSWidget(); 
        
        // Verifica se o click foi aplicado sobre um pedido cinza. Em caso afirmativo, o click deve ser ignorado.
        if(order && order.availableAction !== null){
            
            // Esse caso será executado apenas para os orderOnClick disparados automaticamente devido ao 
            // parametro PLAYANDCONCLUDE (inicia direto na producao)
            if (order && isAutoClick){
                let currentOrder = NavigatorService.getSelectedOrder();
                let lastSelectedOrderIndex = NavigatorService.getOrderIndex(currentOrder);

                if(!(order.processing === true && order.availableAction && order.availableAction.actionLabel.toLowerCase() == 'iniciar')) {
                    order.processing = true;
                    widget.updateKDSTemplate();
                        order.clearBeeps();
                    if (order.availableAction) {

                        if (order.IDSITITPEDFOS == 'E') {
                            clearOrderTimer(order);
                            order.dot = false;
                        }
                        order.availableAction.actionCallback(order);
                        NavigatorService.unSelectAllOrders();
                        NavigatorService.setPreviousOrderAsSelected(++lastSelectedOrderIndex);
                        widget.updateKDSTemplate();
                    }
                }
            // Esse caso será executado para todos os demais cliques efetuados pelo usuário
            } else if (order && !this.isInDelayTime) {

                if(!(order.processing === true && order.availableAction && order.availableAction.actionLabel.toLowerCase() == 'iniciar')) {
                order.processing = true;
                widget.updateKDSTemplate();
                    order.clearBeeps();
                    if (order.availableAction) {

                        this.isInDelayTime = true;
                        setTimeout(()=>{
                            if (order.IDSITITPEDFOS == 'E') {
                                clearOrderTimer(order);	
                                order.dot = false;
                            }
                            order.availableAction.actionCallback(order);
                            this.isInDelayTime = false;
                        }, 100);
                    }
                }
            }
            widget.updateKDSTemplate();
        }
    };

    function clearOrderTimer(order) {
        if (order.timerInterval) {
            $interval.cancel(order.timerInterval);
            delete order.timerInterval;
        }
    }

}

Configuration(function (ContextRegister) {
    ContextRegister.register('OrderActionService', OrderActionService);
});
